#!/usr/bin/env python
# vim: set ts=4 sts=4 sw=4 textwidth=92:

"""
Holds the NoExtract class.

This module works with sets that only need to repair and delete, and do
not need any extraction.

It will detect sets like the following:
X.par2
X.vol0+1.par2
...

01.mp3
02.mp3
...

Where the PAR2 files protect the mp3 files directly.

NOTE: It doesn't just detect mp3's, it will detect any set that does not
NOTE: have any extractable elements in it.
"""

__author__    = "Ira W. Snyder (devel@irasnyder.com)"
__copyright__ = "Copyright (c) 2006,2007 Ira W. Snyder (devel@irasnyder.com)"
__license__   = "GNU GPL v2 (or, at your option, any later version)"

#    NoExtract.py
#
#    Copyright (C) 2006,2007  Ira W. Snyder (devel@irasnyder.com)
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import logging
import PAR2Set.Base
import rsutil.common


def detector (name_files, prot_files):
	"""Detector for the NoExtract type"""

	EXTRACT_REGEX = rsutil.common.config_get_value ('regular expressions', 'extractable_regex')
	return not rsutil.common.has_a_match (EXTRACT_REGEX, prot_files)


class NoExtract (PAR2Set.Base.Base):

	"""Class for sets that do not need to be extracted. Note that this is not a
	   base class, this is a working class."""

	def __repr__ (self):
		return 'NoExtract'

	def runAll (self):
		"""Run the Repair and Deletion stages, omitting the Extraction stage"""

		# Repair Stage
		ret = self.runVerifyAndRepair ()

		if ret != rsutil.common.SUCCESS:
			logging.critical ('Repair stage failed for: %s' % self.p2file)
			return -rsutil.common.ECHECK

		self.update_matches ()

		# Deletion Stage
		ret = self.runDelete ()

		if ret != rsutil.common.SUCCESS:
			logging.critical ('Deletion stage failed for: %s' % self.p2file)
			return -rsutil.common.EDELETE

		logging.info ('Successfully completed: %s' % self.p2file)
		return rsutil.common.SUCCESS

